#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "my_matrix.h"

void dtrsv_(char *UPLO, char *TRANS, char *DIAG, INT * N, double *A, INT *LDA, double *RHS, INT *INCX); 


// Scale a Column 
void scale_col(double s, struct my_matrix_st A, int i, int j, int col){
	int k; 
	for (k=i; k<=j; k++){
		A.values[A.LD*col+k]*=s; 
	}
}

// Rank-1 Update 
void r1_update(struct my_matrix_st A, int i, double s, double *v, int incv, 
	       double *w, int incw) {
	int j,k; 
	for (j = i ; j < A.cols; j++){
		for ( k = i; k< A.rows; k++){
			A.values[k+j*A.LD] += s * v[(k-i)*incv] * w[(j-i)*incw]; 
		}
	}
}

// Compute the Hilbert Matrix  and a sample right hand side
void Hilbert(struct my_matrix_st *H, double *b, int n) {
	int i,j; 
	my_matrix_rand(H,n,n);  /*  Allocation only */
	for (j = 0 ; j < n ; j++) b[j]=0; 
	for (j=0; j<n; j++) {
		for (i = 0 ; i < n ; i ++){
			H->values[H->LD*j+i]= 1.0/(i+1+j); 
			b[i]+=1.0/(i+j+1); 
		}
	}
}

// Insert your code here. 



int main (int argc , char *argv ){
	struct my_matrix_st A; 
	double tic,toc;
	// n is the size of the Hilbert matrix, you can try large n too. 
	int n = 6,  i; 
	double *b; 

	// Build matrix
	b = malloc(sizeof(double)*n); 
	Hilbert(&A, b,n); 
	printf("Input Matrix\n"); 
	my_matrix_print(&A); 


	// Your Code here



	// Print Solution 
	for ( i = 0 ; i < n; i++) {
		printf("b [%3d] = %20.16e\n",i,b[i]); 
	}
	// Cleanup
	my_matrix_clear(&A); 
	free(b); 
	return 0; 

}
